'''
Created on Aug 16, 2016

@author: grastogi
'''
import unittest
from ansible_collections.community.network.plugins.module_utils.network.avi.ansible_utils import \
    cleanup_absent_fields, avi_obj_cmp


class TestAviApiUtils(unittest.TestCase):

    def test_avi_obj_cmp(self):
        obj = {'name': 'testpool'}
        existing_obj = {
            'lb_algorithm': 'LB_ALGORITHM_LEAST_CONNECTIONS',
            'use_service_port': False,
            'server_auto_scale': False,
            'host_check_enabled': False,
            'enabled': True,
            'capacity_estimation': False,
            'fewest_tasks_feedback_delay': 10,
            '_last_modified': '1471377748747040',
            'cloud_ref': 'https://192.0.2.42/api/cloud/cloud-afe8bf2c-9821-4272-9bc6-67634c84bec9',
            'vrf_ref': 'https://192.0.2.42/api/vrfcontext/vrfcontext-0e8ce760-fed2-4650-9397-5b3e4966376e',
            'inline_health_monitor': True,
            'default_server_port': 80,
            'request_queue_depth': 128,
            'graceful_disable_timeout': 1,
            'server_count': 0,
            'sni_enabled': True,
            'request_queue_enabled': False,
            'name': 'testpool',
            'max_concurrent_connections_per_server': 0,
            'url': 'https://192.0.2.42/api/pool/pool-20084ee1-872e-4103-98e1-899103e2242a',
            'tenant_ref': 'https://192.0.2.42/api/tenant/admin',
            'uuid': 'pool-20084ee1-872e-4103-98e1-899103e2242a',
            'connection_ramp_duration': 10}

        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

    def test_avi_obj_cmp_w_refs(self):
        obj = {'name': 'testpool',
               'health_monitor_refs': ['/api/healthmonitor?name=System-HTTP'],
               'enabled': True}
        existing_obj = {
            'lb_algorithm': 'LB_ALGORITHM_LEAST_CONNECTIONS',
            'use_service_port': False,
            'server_auto_scale': False,
            'host_check_enabled': False,
            'enabled': True,
            'capacity_estimation': False,
            'fewest_tasks_feedback_delay': 10,
            '_last_modified': '1471377748747040',
            'cloud_ref': 'https://192.0.2.42/api/cloud/cloud-afe8bf2c-9821-4272-9bc6-67634c84bec9',
            'vrf_ref': 'https://192.0.2.42/api/vrfcontext/vrfcontext-0e8ce760-fed2-4650-9397-5b3e4966376e',
            'inline_health_monitor': True,
            'default_server_port': 80,
            'request_queue_depth': 128,
            'graceful_disable_timeout': 1,
            'server_count': 0,
            'sni_enabled': True,
            'request_queue_enabled': False,
            'name': 'testpool',
            'max_concurrent_connections_per_server': 0,
            'url': 'https://192.0.2.42/api/pool/pool-20084ee1-872e-4103-98e1-899103e2242a',
            'tenant_ref': 'https://192.0.2.42/api/tenant/admin',
            'uuid': 'pool-20084ee1-872e-4103-98e1-899103e2242a',
            'connection_ramp_duration': 10,
            'health_monitor_refs': [
                "https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc9#System-HTTP"],
        }

        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

        obj = {'name': 'testpool',
               'health_monitor_refs': ['/api/healthmonitor?name=System-HTTP'],
               'server_count': 1}
        diff = avi_obj_cmp(obj, existing_obj)
        assert not diff

        obj = {'name': 'testpool',
               'health_monitor_refs': ['api/healthmonitor?name=System-HTTP'],
               'server_count': 0}
        diff = avi_obj_cmp(obj, existing_obj)
        assert not diff
        obj = {'name': 'testpool',
               'health_monitor_refs': ['healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc9'],
               'server_count': 0}
        diff = avi_obj_cmp(obj, existing_obj)
        assert diff
        obj = {'name': 'testpool#asdfasf',
               'health_monitor_refs': ['api/healthmonitor?name=System-HTTP'],
               'server_count': 0}
        diff = avi_obj_cmp(obj, existing_obj)
        assert not diff
        obj = {'name': 'testpool',
               'health_monitor_refs': ['/api/healthmonitor?name=System-HTTP#'],
               'server_count': 0}
        diff = avi_obj_cmp(obj, existing_obj)
        assert not diff

    def test_avi_obj_cmp_empty_list(self):
        obj = {'name': 'testpool',
               'health_monitor_refs': [],
               'enabled': True}
        existing_obj = {
            'lb_algorithm': 'LB_ALGORITHM_LEAST_CONNECTIONS',
            'use_service_port': False,
            'server_auto_scale': False,
            'host_check_enabled': False,
            'enabled': True,
            'capacity_estimation': False,
            'fewest_tasks_feedback_delay': 10,
            '_last_modified': '1471377748747040',
            'cloud_ref': 'https://192.0.2.42/api/cloud/cloud-afe8bf2c-9821-4272-9bc6-67634c84bec9',
            'vrf_ref': 'https://192.0.2.42/api/vrfcontext/vrfcontext-0e8ce760-fed2-4650-9397-5b3e4966376e',
            'inline_health_monitor': True,
            'default_server_port': 80,
            'request_queue_depth': 128,
            'graceful_disable_timeout': 1,
            'server_count': 0,
            'sni_enabled': True,
            'request_queue_enabled': False,
            'name': 'testpool',
            'max_concurrent_connections_per_server': 0,
            'url': 'https://192.0.2.42/api/pool/pool-20084ee1-872e-4103-98e1-899103e2242a',
            'tenant_ref': 'https://192.0.2.42/api/tenant/admin',
            'uuid': 'pool-20084ee1-872e-4103-98e1-899103e2242a',
            'connection_ramp_duration': 10
        }
        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

    def test_avi_obj_cmp_w_refs_n_name(self):
        existing_obj = {
            'use_service_port': False,
            'server_auto_scale': False,
            'host_check_enabled': False,
            'enabled': True,
            'capacity_estimation': False,
            'fewest_tasks_feedback_delay': 10,
            '_last_modified': '1471377748747040',
            'cloud_ref': 'https://192.0.2.42/api/cloud/cloud-afe8bf2c-9821-4272-9bc6-67634c84bec9',
            'vrf_ref': 'https://192.0.2.42/api/vrfcontext/vrfcontext-0e8ce760-fed2-4650-9397-5b3e4966376e',
            'inline_health_monitor': True,
            'default_server_port': 80,
            'request_queue_depth': 128,
            'graceful_disable_timeout': 1,
            'server_count': 0,
            'sni_enabled': True,
            'request_queue_enabled': False,
            'name': 'testpool',
            'max_concurrent_connections_per_server': 0,
            'url': 'https://192.0.2.42/api/pool/pool-20084ee1-872e-4103-98e1-899103e2242a',
            'tenant_ref': 'https://192.0.2.42/api/tenant/admin',
            'uuid': 'pool-20084ee1-872e-4103-98e1-899103e2242a',
            'connection_ramp_duration': 10,
            'health_monitor_refs': [
                "https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc9#System-HTTP",
                "https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc8",
            ],
        }

        obj = {'name': 'testpool',
               'health_monitor_refs': ['https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc9',
                                       "https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc8"],
               'server_count': 0}
        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

        obj = {'name': 'testpool',
               'health_monitor_refs': [
                   'https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc9#System-HTTP',
                   "https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc8"],
               'server_count': 0}
        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

        obj = {'name': 'testpool',
               'health_monitor_refs': [
                   'https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc9#System-HTTP',
                   "https://192.0.2.42/api/healthmonitor/healthmonitor-6d07b57f-126b-476c-baba-a8c8c8b06dc8#System-HTTP2"],
               'server_count': 0,
               'cloud_ref': 'https://192.0.2.42/api/cloud/cloud-afe8bf2c-9821-4272-9bc6-67634c84bec9#Default-Cloud',
               }
        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

    def test_avi_list_update(self):
        existing_obj = {
            'services': [
                {
                    "enable_ssl": False,
                    "port_range_end": 80,
                    "port": 80
                },
                {
                    "enable_ssl": False,
                    "port_range_end": 443,
                    "port": 443
                }
            ],
            "name": "vs-health-test",
            "url": "https://192.0.2.42/api/virtualservice/virtualservice-526c55c2-df89-40b9-9de6-e45a472290aa",
        }

        obj = {
            'services': [
                {
                    "enable_ssl": False,
                    "port_range_end": 80,
                    "port": 80
                }
            ]
        }

        diff = avi_obj_cmp(obj, existing_obj)
        assert not diff

        obj = {
            'services': [
                {
                    "enable_ssl": False,
                    "port_range_end": 80,
                    "port": 80
                },
                {
                    "enable_ssl": False,
                    "port_range_end": 443,
                    "port": 80
                }
            ],
            "name": "vs-health-test",
            "url": "https://192.0.2.42/api/virtualservice/virtualservice-526c55c2-df89-40b9-9de6-e45a472290aa",
        }

        diff = avi_obj_cmp(obj, existing_obj)
        assert not diff

    def test_cleanup_abset(self):
        obj = {'x': 10,
               'y': {'state': 'absent'},
               'z': {'a': {'state': 'absent'}},
               'l': [{'y1': {'state': 'absent'}}],
               'z1': {'a': {'state': 'absent'}, 'b': {}, 'c': 42},
               'empty': []}

        obj = cleanup_absent_fields(obj)

        assert 'y' not in obj
        assert 'z' not in obj
        assert 'l' not in obj
        assert 'z1' in obj
        assert 'b' not in obj['z1']
        assert 'a' not in obj['z1']
        assert 'empty' not in obj

    def test_complex_obj(self):

        obj = {
            'lb_algorithm': 'LB_ALGORITHM_ROUND_ROBIN',
            'use_service_port': False, 'server_auto_scale': False,
            'host_check_enabled': False,
            'tenant_ref': 'https://192.0.2.42/api/tenant/admin#admin',
            'capacity_estimation': False,
            'servers': [{
                'hostname': 'grastogi-server6', 'ratio': 1,
                'ip': {'type': 'V4', 'addr': '198.51.100.62'},
                'discovered_networks': [{
                    'subnet': [{
                        'ip_addr': {
                            'type': 'V4',
                            'addr': '198.51.100.0'
                        },
                        'mask': 24
                    }],
                    'network_ref': 'https://192.0.2.42/api/network/dvportgroup-53975-10.10.2.10#PG-964'
                }],
                'enabled': True, 'nw_ref': 'https://192.0.2.42/api/vimgrnwruntime/dvportgroup-53975-10.10.2.10#PG-964',
                'verify_network': False,
                'static': False,
                'resolve_server_by_dns': False,
                'external_uuid': 'vm-4230615e-bc0b-3d33-3929-1c7328575993',
                'vm_ref': 'https://192.0.2.42/api/vimgrvmruntime/vm-4230615e-bc0b-3d33-3929-1c7328575993#grastogi-server6'
            }, {
                'hostname': 'grastogi-server6',
                'ratio': 1,
                'ip': {
                    'type': 'V4',
                    'addr': '198.51.100.61'
                },
                'discovered_networks': [{
                    'subnet': [{
                        'ip_addr': {
                            'type': 'V4',
                            'addr': '198.51.100.0'
                        },
                        'mask': 24
                    }],
                    'network_ref': 'https://192.0.2.42/api/network/dvportgroup-53975-10.10.2.10#PG-964'
                }],
                'enabled': True,
                'nw_ref': 'https://192.0.2.42/api/vimgrnwruntime/dvportgroup-53975-10.10.2.10#PG-964',
                'verify_network': False,
                'static': False,
                'resolve_server_by_dns': False,
                'external_uuid': 'vm-4230615e-bc0b-3d33-3929-1c7328575993',
                'vm_ref': 'https://192.0.2.42/api/vimgrvmruntime/vm-4230615e-bc0b-3d33-3929-1c7328575993#grastogi-server6'
            }, {
                'hostname': 'grastogi-server6',
                'ratio': 1,
                'ip': {
                    'type': 'V4',
                    'addr': '198.51.100.65'
                },
                'discovered_networks': [{
                    'subnet': [{
                        'ip_addr': {
                            'type': 'V4',
                            'addr': '198.51.100.0'
                        }, 'mask': 24
                    }],
                    'network_ref': 'https://192.0.2.42/api/network/dvportgroup-53975-10.10.2.10#PG-964'
                }],
                'enabled': True,
                'verify_network': False,
                'static': False,
                'resolve_server_by_dns': False
            }],
            'fewest_tasks_feedback_delay': 10,
            '_last_modified': '1473292763246107',
            'cloud_ref': 'https://192.0.2.42/api/cloud/cloud-e0696a58-8b72-4026-923c-9a87c38a2489#Default-Cloud',
            'vrf_ref': 'https://192.0.2.42/api/vrfcontext/vrfcontext-33dfbcd7-867c-4e3e-acf7-96bf679d5a0d#global',
            'inline_health_monitor': True,
            'default_server_port': 8000,
            'request_queue_depth': 128,
            'graceful_disable_timeout': 1,
            'sni_enabled': True,
            'server_count': 3,
            'uuid': 'pool-09201181-747e-41ea-872d-e9a7df71b726',
            'request_queue_enabled': False,
            'name': 'p1',
            'max_concurrent_connections_per_server': 0,
            'url': 'https://192.0.2.42/api/pool/pool-09201181-747e-41ea-872d-e9a7df71b726#p1',
            'enabled': True,
            'connection_ramp_duration': 10}

        existing_obj = {
            'lb_algorithm': 'LB_ALGORITHM_ROUND_ROBIN',
            'use_service_port': False,
            'server_auto_scale': False,
            'host_check_enabled': False,
            'tenant_ref': 'https://192.0.2.42/api/tenant/admin',
            'capacity_estimation': False,
            'servers': [{
                'hostname': 'grastogi-server6', 'ratio': 1,
                'ip': {
                    'type': 'V4',
                    'addr': '198.51.100.62'
                },
                'discovered_networks': [{
                    'subnet': [{
                        'mask': 24,
                        'ip_addr': {
                            'type': 'V4',
                            'addr': '198.51.100.0'
                        }
                    }],
                    'network_ref': 'https://192.0.2.42/api/network/dvportgroup-53975-10.10.2.10'
                }],
                'enabled': True,
                'nw_ref': 'https://192.0.2.42/api/vimgrnwruntime/dvportgroup-53975-10.10.2.10',
                'verify_network': False,
                'static': False,
                'resolve_server_by_dns': False,
                'external_uuid': 'vm-4230615e-bc0b-3d33-3929-1c7328575993',
                'vm_ref': 'https://192.0.2.42/api/vimgrvmruntime/vm-4230615e-bc0b-3d33-3929-1c7328575993'
            }, {
                'hostname': 'grastogi-server6',
                'ratio': 1,
                'ip': {
                    'type': 'V4',
                    'addr': '198.51.100.61'
                },
                'discovered_networks': [{
                    'subnet': [{
                        'mask': 24,
                        'ip_addr': {
                            'type': 'V4',
                            'addr': '198.51.100.0'
                        }
                    }],
                    'network_ref': 'https://192.0.2.42/api/network/dvportgroup-53975-10.10.2.10'
                }],
                'enabled': True,
                'nw_ref': 'https://192.0.2.42/api/vimgrnwruntime/dvportgroup-53975-10.10.2.10',
                'verify_network': False,
                'static': False,
                'resolve_server_by_dns': False,
                'external_uuid': 'vm-4230615e-bc0b-3d33-3929-1c7328575993',
                'vm_ref': 'https://192.0.2.42/api/vimgrvmruntime/vm-4230615e-bc0b-3d33-3929-1c7328575993'
            }, {
                'hostname': 'grastogi-server6',
                'ratio': 1,
                'ip': {
                    'type': 'V4',
                    'addr': '198.51.100.65'
                },
                'discovered_networks': [{
                    'subnet': [{
                        'mask': 24,
                        'ip_addr': {
                            'type': 'V4',
                            'addr': '198.51.100.0'
                        }
                    }],
                    'network_ref': 'https://192.0.2.42/api/network/dvportgroup-53975-10.10.2.10'
                }],
                'enabled': True,
                'nw_ref': 'https://192.0.2.42/api/vimgrnwruntime/dvportgroup-53975-10.10.2.10',
                'verify_network': False,
                'static': False,
                'resolve_server_by_dns': False,
                'external_uuid': 'vm-4230615e-bc0b-3d33-3929-1c7328575993',
                'vm_ref': 'https://192.0.2.42/api/vimgrvmruntime/vm-4230615e-bc0b-3d33-3929-1c7328575993'
            }],
            'fewest_tasks_feedback_delay': 10,
            'cloud_ref': 'https://192.0.2.42/api/cloud/cloud-e0696a58-8b72-4026-923c-9a87c38a2489',
            'vrf_ref': 'https://192.0.2.42/api/vrfcontext/vrfcontext-33dfbcd7-867c-4e3e-acf7-96bf679d5a0d',
            'inline_health_monitor': True,
            'default_server_port': 8000,
            'request_queue_depth': 128,
            'graceful_disable_timeout': 1,
            'sni_enabled': True,
            'server_count': 3,
            'uuid': 'pool-09201181-747e-41ea-872d-e9a7df71b726',
            'request_queue_enabled': False,
            'name': 'p1',
            'max_concurrent_connections_per_server': 0,
            'url': 'https://192.0.2.42/api/pool/pool-09201181-747e-41ea-872d-e9a7df71b726',
            'enabled': True,
            'connection_ramp_duration': 10
        }

        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

    def testAWSVs(self):
        existing_obj = {
            'network_profile_ref': 'https://12.97.16.202/api/networkprofile/networkprofile-9a0a9896-6876-44c8-a3ee-512a968905f2#System-TCP-Proxy',
            'port_uuid': 'eni-4144e73c',
            'weight': 1,
            'availability_zone': 'us-west-2a',
            'enabled': True,
            'flow_dist': 'LOAD_AWARE',
            'subnet_uuid': 'subnet-91f0b6f4',
            'delay_fairness': False,
            'avi_allocated_vip': True,
            'vrf_context_ref': 'https://12.97.16.202/api/vrfcontext/vrfcontext-722b280d-b555-4d82-9b35-af9442c0cb86#global',
            'subnet': {
                'ip_addr': {
                    'type': 'V4',
                    'addr': '198.51.100.0'
                },
                'mask': 24
            },
            'cloud_type': 'CLOUD_AWS', 'uuid': 'virtualservice-a5f49b99-22c8-42e6-aa65-3ca5f1e36b9e',
            'network_ref': 'https://12.97.16.202/api/network/subnet-91f0b6f4',
            'cloud_ref': 'https://12.97.16.202/api/cloud/cloud-49829414-c704-43ca-9dff-05b9e8474dcb#AWS Cloud',
            'avi_allocated_fip': False,
            'se_group_ref': 'https://12.97.16.202/api/serviceenginegroup/serviceenginegroup-3bef6320-5a2d-4801-85c4-ef4f9841f235#Default-Group',
            'scaleout_ecmp': False,
            'max_cps_per_client': 0,
            'type': 'VS_TYPE_NORMAL',
            'analytics_profile_ref': 'https://12.97.16.202/api/analyticsprofile/analyticsprofile-70f8b06f-7b6a-4500-b829-c869bbca2009#System-Analytics-Profile',
            'use_bridge_ip_as_vip': False,
            'application_profile_ref': 'https://12.97.16.202/api/applicationprofile/applicationprofile-103cbc31-cac5-46ab-8e66-bbbb2c8f551f#System-HTTP',
            'auto_allocate_floating_ip': False,
            'services': [{
                'enable_ssl': False,
                'port_range_end': 80,
                'port': 80
            }],
            'active_standby_se_tag': 'ACTIVE_STANDBY_SE_1',
            'ip_address': {
                'type': 'V4',
                'addr': '198.51.100.33'
            },
            'ign_pool_net_reach': False,
            'east_west_placement': False,
            'limit_doser': False,
            'name': 'wwwawssit.ebiz.verizon.com',
            'url': 'https://12.97.16.202/api/virtualservice/virtualservice-a5f49b99-22c8-42e6-aa65-3ca5f1e36b9e#wwwawssit.ebiz.verizon.com',
            'ssl_sess_cache_avg_size': 1024,
            'enable_autogw': True,
            'auto_allocate_ip': True,
            'tenant_ref': 'https://12.97.16.202/api/tenant/tenant-f52f7a3e-6876-4bb9-b8f7-3cab636dadf2#Sales',
            'remove_listening_port_on_vs_down': False
        }
        obj = {'auto_allocate_ip': True, 'subnet_uuid': 'subnet-91f0b6f4', 'cloud_ref': '/api/cloud?name=AWS Cloud', 'services': [{'port': 80}],
               'name': 'wwwawssit.ebiz.verizon.com'}

        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

    def testhttppolicy(self):
        existing_obj = {
            "http_request_policy": {
                "rules": [{
                    "enable": True,
                    "index": 0,
                    "match": {
                        "path": {
                            "match_case": "INSENSITIVE",
                            "match_criteria": "CONTAINS",
                            "match_str": ["xvz", "rst"]
                        }
                    },
                    "name": "blah",
                    "switching_action": {
                        "action": "HTTP_SWITCHING_SELECT_POOL",
                        "pool_ref": "https://12.97.16.202/api/pool/pool-d7f6f5e7-bd26-49ad-aeed-965719eb140b#abc",
                        "status_code": "HTTP_LOCAL_RESPONSE_STATUS_CODE_200"
                    }
                }]
            },
            "is_internal_policy": False,
            "name": "blah",
            "tenant_ref": "https://12.97.16.202/api/tenant/tenant-f52f7a3e-6876-4bb9-b8f7-3cab636dadf2#Sales",
            "url": "https://12.97.16.202/api/httppolicyset/httppolicyset-ffd8354b-671b-48d5-92cc-69a9057aad0c#blah",
            "uuid": "httppolicyset-ffd8354b-671b-48d5-92cc-69a9057aad0c"
        }

        obj = {
            "http_request_policy": {
                "rules": [{
                    "enable": True,
                    "index": "0",
                    "match": {
                        "path": {
                            "match_case": "INSENSITIVE",
                            "match_criteria": "CONTAINS",
                            "match_str": ["xvz", "rst"]
                        }
                    },
                    "name": "blah",
                    "switching_action": {
                        "action": "HTTP_SWITCHING_SELECT_POOL",
                        "pool_ref": "/api/pool?name=abc",
                        "status_code": "HTTP_LOCAL_RESPONSE_STATUS_CODE_200"
                    }
                }]
            },
            "is_internal_policy": False,
            "tenant": "Sales"
        }
        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

    def testCleanupFields(self):
        obj = {'name': 'testpool',
               'scalar_field': {'state': 'absent'},
               'list_fields': [{'x': '1'}, {'y': {'state': 'absent'}}]}

        cleanup_absent_fields(obj)
        assert 'scalar_field' not in obj
        for elem in obj['list_fields']:
            assert 'y' not in elem

    def testGSLB(self):
        obj = {
            'domain_names': ['cloud5.avi.com', 'cloud6.avi.com'],
            'health_monitor_scope': 'GSLB_SERVICE_HEALTH_MONITOR_ALL_MEMBERS',
            'groups': [{
                'priority': 20,
                'members': [{
                    'ip': {
                        'type': 'V4',
                        'addr': '198.51.100.1'
                    },
                    'enabled': True, 'ratio': 1
                }, {
                    'ip': {
                        'type': 'V4',
                        'addr': '198.51.100.10'
                    },
                    'enabled': True,
                    'ratio': 1
                }],
                'algorithm': 'GSLB_ALGORITHM_CONSISTENT_HASH',
                'name': 'sc'
            }, {
                'priority': 14,
                'members': [{
                    'ip': {
                        'type': 'V4',
                        'addr': '198.51.100.2'
                    },
                    'enabled': True,
                    'ratio': 1
                }],
                'algorithm': 'GSLB_ALGORITHM_ROUND_ROBIN',
                'name': 'cn'
            }, {
                'priority': 15,
                'members': [{
                    'ip': {
                        'type': 'V4',
                        'addr': '198.51.100.3'
                    },
                    'enabled': True, 'ratio': 1
                }],
                'algorithm': 'GSLB_ALGORITHM_ROUND_ROBIN',
                'name': 'in'
            }],
            'name': 'gs-3',
            'num_dns_ip': 2
        }
        existing_obj = {
            u'controller_health_status_enabled': True,
            u'uuid': u'gslbservice-ab9b36bd-3e95-4c2e-80f8-92905c2eccb2',
            u'wildcard_match': False,
            u'url': u'https://192.0.2.42/api/gslbservice/gslbservice-ab9b36bd-3e95-4c2e-80f8-92905c2eccb2#gs-3',
            u'tenant_ref': u'https://192.0.2.42/api/tenant/admin#admin',
            u'enabled': True,
            u'domain_names': [u'cloud5.avi.com', u'cloud6.avi.com'],
            u'use_edns_client_subnet': True,
            u'groups': [{
                u'priority': 20,
                u'members': [{
                    u'ip': {
                        u'type': u'V4',
                        u'addr': u'198.51.100.1'
                    },
                    u'ratio': 1,
                    u'enabled': True
                }, {
                    u'ip': {
                        u'type': u'V4',
                        u'addr': u'198.51.100.10'
                    },
                    u'ratio': 1,
                    u'enabled': True
                }],
                u'name': u'sc',
                u'algorithm': u'GSLB_ALGORITHM_CONSISTENT_HASH'
            }, {
                u'priority': 14,
                u'members': [{
                    u'ip': {
                        u'type': u'V4',
                        u'addr': u'198.51.100.2'
                    },
                    u'ratio': 1,
                    u'enabled': True
                }],
                u'name': u'cn',
                u'algorithm': u'GSLB_ALGORITHM_ROUND_ROBIN'
            }, {
                u'priority': 15,
                u'members': [{
                    u'ip': {
                        u'type': u'V4',
                        u'addr': u'198.51.100.3'
                    },
                    u'ratio': 1,
                    u'enabled': True
                }],
                u'name': u'in',
                u'algorithm': u'GSLB_ALGORITHM_ROUND_ROBIN'
            }],
            u'num_dns_ip': 2,
            u'health_monitor_scope': u'GSLB_SERVICE_HEALTH_MONITOR_ALL_MEMBERS',
            u'name': u'gs-3'
        }
        diff = avi_obj_cmp(obj, existing_obj)
        assert diff

    def testNoneParams(self):
        objwnone = {
            'name': 'testpool',
            'scalar_field': None,
            'list_fields': {
                'y': None,
                'z': 'zz'
            }
        }
        obj = {
            'name': 'testpool',
            'list_fields': {
                'z': 'zz'
            }
        }

        result = avi_obj_cmp(objwnone, obj)
        assert result



#!/usr/bin/python
# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: pn_dscp_map
author: "Pluribus Networks (@rajaspachipulusu17)"
short_description: CLI command to create/delete dscp-map
description:
  - This module can be used to create a DSCP priority mapping table.
options:
  pn_cliswitch:
    description:
      - Target switch to run the CLI on.
    required: False
    type: str
  state:
    description:
      - State the action to perform. Use C(present) to create dscp-map and
        C(absent) to delete.
    required: True
    type: str
    choices: ["present", "absent"]
  pn_name:
    description:
      - Name for the DSCP map.
    required: False
    type: str
  pn_scope:
    description:
      - Scope for dscp map.
    required: False
    choices: ["local", "fabric"]
'''

EXAMPLES = """
- name: Dscp map create
  community.network.pn_dscp_map:
    pn_cliswitch: "sw01"
    state: "present"
    pn_name: "foo"
    pn_scope: "local"

- name: Dscp map delete
  community.network.pn_dscp_map:
    pn_cliswitch: "sw01"
    state: "absent"
    pn_name: "foo"
"""

RETURN = """
command:
  description: the CLI command run on the target node.
  returned: always
  type: str
stdout:
  description: set of responses from the dscp-map command.
  returned: always
  type: list
stderr:
  description: set of error responses from the dscp-map command.
  returned: on error
  type: list
changed:
  description: indicates whether the CLI caused changes on the target.
  returned: always
  type: bool
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.netvisor.pn_nvos import pn_cli, run_cli
from ansible_collections.community.network.plugins.module_utils.network.netvisor.netvisor import run_commands


def check_cli(module, cli):
    """
    This method checks for idempotency using the dscp-map-show name command.
    If a user with given name exists, return True else False.
    :param module: The Ansible module to fetch input parameters
    :param cli: The CLI string
    """
    name = module.params['pn_name']

    cli += ' dscp-map-show format name no-show-headers'
    out = run_commands(module, cli)[1]

    if out:
        out = out.split()

    return True if name in out else False


def main():
    """ This section is for arguments parsing """

    state_map = dict(
        present='dscp-map-create',
        absent='dscp-map-delete'
    )

    module = AnsibleModule(
        argument_spec=dict(
            pn_cliswitch=dict(required=False, type='str'),
            state=dict(required=True, type='str',
                       choices=state_map.keys()),
            pn_name=dict(required=False, type='str'),
            pn_scope=dict(required=False, type='str',
                          choices=['local', 'fabric']),
        ),
        required_if=(
            ["state", "present", ["pn_name", "pn_scope"]],
            ["state", "absent", ["pn_name"]],
        )
    )

    # Accessing the arguments
    cliswitch = module.params['pn_cliswitch']
    state = module.params['state']
    name = module.params['pn_name']
    scope = module.params['pn_scope']

    command = state_map[state]

    # Building the CLI command string
    cli = pn_cli(module, cliswitch)

    NAME_EXISTS = check_cli(module, cli)
    cli += ' %s name %s ' % (command, name)

    if command == 'dscp-map-delete':
        if NAME_EXISTS is False:
            module.exit_json(
                skipped=True,
                msg='dscp map with name %s does not exist' % name
            )
    else:
        if command == 'dscp-map-create':
            if NAME_EXISTS is True:
                module.exit_json(
                    skipped=True,
                    msg='dscp map with name %s already exists' % name
                )

        if scope:
            cli += ' scope ' + scope

    run_cli(module, cli, state_map)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: pn_vrouter_loopback_interface
author: "Pluribus Networks (@rajaspachipulusu17)"
short_description: CLI command to add/remove vrouter-loopback-interface
description:
  - This module can be used to add loopback interface to a vRouter or
    remove loopback interface from a vRouter.
options:
  pn_cliswitch:
    description:
      - Target switch to run the CLI on.
    required: false
    type: str
  state:
    description:
      - State the action to perform. Use C(present) to add vrouter-loopback-interface
        and C(absent) to remove vrouter-loopback-interface.
    required: false
    type: str
    choices: ['present', 'absent']
    default: 'present'
  pn_ip:
    description:
      - loopback IP address.
    required: true
    type: str
  pn_index:
    description:
      - loopback index from 1 to 255.
    required: false
    type: str
  pn_vrouter_name:
    description:
      - name of service config.
    required: true
    type: str
'''

EXAMPLES = """
- name: Add vrouter loopback interface
  community.network.pn_vrouter_loopback_interface:
    state: "present"
    pn_cliswitch: "sw01"
    pn_vrouter_name: "sw01-vrouter"
    pn_ip: "192.168.10.1"

- name: Remove vrouter loopback interface
  community.network.pn_vrouter_loopback_interface:
    state: "absent"
    pn_cliswitch: "sw01"
    pn_vrouter_name: "sw01-vrouter"
    pn_ip: "192.168.10.1"
    pn_index: "2"
"""

RETURN = """
command:
  description: the CLI command run on the target node.
  returned: always
  type: str
stdout:
  description: set of responses from the vrouter-loopback-interface command.
  returned: always
  type: list
stderr:
  description: set of error response from the vrouter-loopback-interface
               command.
  returned: on error
  type: list
changed:
  description: indicates whether the CLI caused changes on the target.
  returned: always
  type: bool
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.netvisor.pn_nvos import pn_cli, run_cli
from ansible_collections.community.network.plugins.module_utils.network.netvisor.netvisor import run_commands


def check_cli(module, cli):
    """
    This method checks if vRouter exists on the target node.
    This method also checks for idempotency using the vrouter-interface-show
    command.
    If the given vRouter exists, return VROUTER_EXISTS as True else False.

    :param module: The Ansible module to fetch input parameters
    :param cli: The CLI string
    :return Booleans: VROUTER_EXISTS, INTERFACE_EXISTS
    """
    vrouter_name = module.params['pn_vrouter_name']
    interface_ip = module.params['pn_ip']

    # Check for vRouter
    check_vrouter = 'vrouter-show format name no-show-headers'
    out = run_commands(module, check_vrouter)[1]
    if out:
        out = out.split()

    VROUTER_EXISTS = True if vrouter_name in out else False

    if interface_ip:
        # Check for interface and VRRP and fetch nic for VRRP
        show = cli + ' vrouter-loopback-interface-show '
        show += 'vrouter-name %s ' % vrouter_name
        show += 'format ip no-show-headers'
        out = run_commands(module, show)[1]

        if out and interface_ip in out.split():
            INTERFACE_EXISTS = True
        else:
            INTERFACE_EXISTS = False

    return VROUTER_EXISTS, INTERFACE_EXISTS


def main():
    """ This section is for arguments parsing """

    state_map = dict(
        present='vrouter-loopback-interface-add',
        absent='vrouter-loopback-interface-remove'
    )

    argument_spec = dict(
        pn_cliswitch=dict(required=False, type='str'),
        state=dict(required=False, type='str',
                   choices=state_map.keys(), default='present'),
        pn_ip=dict(required=True, type='str'),
        pn_index=dict(required=False, type='str'),
        pn_vrouter_name=dict(required=True, type='str'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=(
            ["state", "present", ["pn_vrouter_name", "pn_ip"]],
            ["state", "absent", ["pn_vrouter_name", "pn_ip", "pn_index"]]
        ),
    )

    # Accessing the arguments
    cliswitch = module.params['pn_cliswitch']
    state = module.params['state']
    ip = module.params['pn_ip']
    index = module.params['pn_index']
    vrouter_name = module.params['pn_vrouter_name']

    command = state_map[state]

    # Building the CLI command string
    cli = pn_cli(module, cliswitch)

    VROUTER_EXISTS, INTERFACE_EXISTS = check_cli(module, cli)
    cli += ' %s vrouter-name %s ' % (command, vrouter_name)

    if index and (int(index) < 1 or int(index) > 255):
        module.fail_json(
            failed=True,
            msg='index should be in range 1 to 255'
        )

    if index and state == 'present':
        show = 'vrouter-loopback-interface-show format index parsable-delim ,'
        out = run_commands(module, show)[1]
        if out:
            out = out.split()
            for res in out:
                res = res.strip().split(',')
                if index in res:
                    module.fail_json(
                        failed=True,
                        msg='index with value %s exist' % index
                    )

    if command == 'vrouter-loopback-interface-add':
        if VROUTER_EXISTS is False:
            module.fail_json(
                failed=True,
                msg='vRouter %s does not exist' % vrouter_name
            )
        if INTERFACE_EXISTS is True:
            module.exit_json(
                skipped=True,
                msg='vRouter with loopback ip %s exist' % ip
            )
        if ip:
            cli += ' ip ' + ip
        if index:
            cli += ' index ' + index

    if command == 'vrouter-loopback-interface-remove':
        if VROUTER_EXISTS is False:
            module.fail_json(
                failed=True,
                msg='vRouter %s does not exist' % vrouter_name
            )
        if INTERFACE_EXISTS is False:
            module.exit_json(
                skipped=True,
                msg='vRouter with loopback ip %s doesnt exist' % ip
            )

        if index:
            cli += ' index ' + index

    run_cli(module, cli, state_map)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2018, Chris Houseknecht <@chouseknecht>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = r'''

module: k8s

short_description: Manage Kubernetes (K8s) objects

author:
    - "Chris Houseknecht (@chouseknecht)"
    - "Fabian von Feilitzsch (@fabianvf)"

description:
  - Use the OpenShift Python client to perform CRUD operations on K8s objects.
  - Pass the object definition from a source file or inline. See examples for reading
    files and using Jinja templates or vault-encrypted files.
  - Access to the full range of K8s APIs.
  - Use the M(community.kubernetes.k8s_info) module to obtain a list of items about an object of type C(kind)
  - Authenticate using either a config file, certificates, password or token.
  - Supports check mode.

extends_documentation_fragment:
  - community.kubernetes.k8s_state_options
  - community.kubernetes.k8s_name_options
  - community.kubernetes.k8s_resource_options
  - community.kubernetes.k8s_auth_options

notes:
  - If your OpenShift Python library is not 0.9.0 or newer and you are trying to
    remove an item from an associative array/dictionary, for example a label or
    an annotation, you will need to explicitly set the value of the item to be
    removed to `null`. Simply deleting the entry in the dictionary will not
    remove it from openshift or kubernetes.

options:
  merge_type:
    description:
    - Whether to override the default patch merge approach with a specific type. By default, the strategic
      merge will typically be used.
    - For example, Custom Resource Definitions typically aren't updatable by the usual strategic merge. You may
      want to use C(merge) if you see "strategic merge patch format is not supported"
    - See U(https://kubernetes.io/docs/tasks/run-application/update-api-object-kubectl-patch/#use-a-json-merge-patch-to-update-a-deployment)
    - Requires openshift >= 0.6.2
    - If more than one merge_type is given, the merge_types will be tried in order
    - If openshift >= 0.6.2, this defaults to C(['strategic-merge', 'merge']), which is ideal for using the same parameters
      on resource kinds that combine Custom Resources and built-in resources. For openshift < 0.6.2, the default
      is simply C(strategic-merge).
    - mutually exclusive with C(apply)
    choices:
    - json
    - merge
    - strategic-merge
    type: list
    elements: str
  wait:
    description:
    - Whether to wait for certain resource kinds to end up in the desired state. By default the module exits once Kubernetes has
      received the request
    - Implemented for C(state=present) for C(Deployment), C(DaemonSet) and C(Pod), and for C(state=absent) for all resource kinds.
    - For resource kinds without an implementation, C(wait) returns immediately unless C(wait_condition) is set.
    default: no
    type: bool
  wait_sleep:
    description:
    - Number of seconds to sleep between checks.
    default: 5
    type: int
  wait_timeout:
    description:
    - How long in seconds to wait for the resource to end up in the desired state. Ignored if C(wait) is not set.
    default: 120
    type: int
  wait_condition:
    description:
    - Specifies a custom condition on the status to wait for. Ignored if C(wait) is not set or is set to False.
    suboptions:
      type:
        type: str
        description:
        - The type of condition to wait for. For example, the C(Pod) resource will set the C(Ready) condition (among others)
        - Required if you are specifying a C(wait_condition). If left empty, the C(wait_condition) field will be ignored.
        - The possible types for a condition are specific to each resource type in Kubernetes. See the API documentation of the status field
          for a given resource to see possible choices.
      status:
        type: str
        description:
        - The value of the status field in your desired condition.
        - For example, if a C(Deployment) is paused, the C(Progressing) C(type) will have the C(Unknown) status.
        choices:
        - True
        - False
        - Unknown
        default: "True"
      reason:
        type: str
        description:
        - The value of the reason field in your desired condition
        - For example, if a C(Deployment) is paused, The C(Progressing) C(type) will have the C(DeploymentPaused) reason.
        - The possible reasons in a condition are specific to each resource type in Kubernetes. See the API documentation of the status field
          for a given resource to see possible choices.
    type: dict
  validate:
    description:
      - how (if at all) to validate the resource definition against the kubernetes schema.
        Requires the kubernetes-validate python module and openshift >= 0.8.0
    suboptions:
      fail_on_error:
        description: whether to fail on validation errors.
        type: bool
      version:
        description: version of Kubernetes to validate against. defaults to Kubernetes server version
        type: str
      strict:
        description: whether to fail when passing unexpected properties
        default: True
        type: bool
    type: dict
  append_hash:
    description:
    - Whether to append a hash to a resource name for immutability purposes
    - Applies only to ConfigMap and Secret resources
    - The parameter will be silently ignored for other resource kinds
    - The full definition of an object is needed to generate the hash - this means that deleting an object created with append_hash
      will only work if the same object is passed with state=absent (alternatively, just use state=absent with the name including
      the generated hash and append_hash=no)
    - Requires openshift >= 0.7.2
    type: bool
  apply:
    description:
    - C(apply) compares the desired resource definition with the previously supplied resource definition,
      ignoring properties that are automatically generated
    - C(apply) works better with Services than 'force=yes'
    - Requires openshift >= 0.9.2
    - mutually exclusive with C(merge_type)
    type: bool

requirements:
  - "python >= 2.7"
  - "openshift >= 0.6"
  - "PyYAML >= 3.11"
'''

EXAMPLES = r'''
- name: Create a k8s namespace
  community.kubernetes.k8s:
    name: testing
    api_version: v1
    kind: Namespace
    state: present

- name: Create a Service object from an inline definition
  community.kubernetes.k8s:
    state: present
    definition:
      apiVersion: v1
      kind: Service
      metadata:
        name: web
        namespace: testing
        labels:
          app: galaxy
          service: web
      spec:
        selector:
          app: galaxy
          service: web
        ports:
        - protocol: TCP
          targetPort: 8000
          name: port-8000-tcp
          port: 8000

- name: Remove an existing Service object
  community.kubernetes.k8s:
    state: absent
    api_version: v1
    kind: Service
    namespace: testing
    name: web

# Passing the object definition from a file

- name: Create a Deployment by reading the definition from a local file
  community.kubernetes.k8s:
    state: present
    src: /testing/deployment.yml

- name: >-
    Read definition file from the Ansible controller file system.
    If the definition file has been encrypted with Ansible Vault it will automatically be decrypted.
  community.kubernetes.k8s:
    state: present
    definition: "{{ lookup('file', '/testing/deployment.yml') | from_yaml }}"

- name: Read definition file from the Ansible controller file system after Jinja templating
  community.kubernetes.k8s:
    state: present
    definition: "{{ lookup('template', '/testing/deployment.yml') | from_yaml }}"

- name: fail on validation errors
  community.kubernetes.k8s:
    state: present
    definition: "{{ lookup('template', '/testing/deployment.yml') | from_yaml }}"
    validate:
      fail_on_error: yes

- name: warn on validation errors, check for unexpected properties
  community.kubernetes.k8s:
    state: present
    definition: "{{ lookup('template', '/testing/deployment.yml') | from_yaml }}"
    validate:
      fail_on_error: no
      strict: yes
'''

RETURN = r'''
result:
  description:
  - The created, patched, or otherwise present object. Will be empty in the case of a deletion.
  returned: success
  type: complex
  contains:
     api_version:
       description: The versioned schema of this representation of an object.
       returned: success
       type: str
     kind:
       description: Represents the REST resource this object represents.
       returned: success
       type: str
     metadata:
       description: Standard object metadata. Includes name, namespace, annotations, labels, etc.
       returned: success
       type: complex
     spec:
       description: Specific attributes of the object. Will vary based on the I(api_version) and I(kind).
       returned: success
       type: complex
     status:
       description: Current status details for the object.
       returned: success
       type: complex
     items:
       description: Returned only when multiple yaml documents are passed to src or resource_definition
       returned: when resource_definition or src contains list of objects
       type: list
     duration:
       description: elapsed time of task in seconds
       returned: when C(wait) is true
       type: int
       sample: 48
'''

from ansible_collections.community.kubernetes.plugins.module_utils.raw import KubernetesRawModule


def main():
    KubernetesRawModule().execute_module()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function

__metaclass__ = type
DOCUMENTATION = """
module: ios_lldp
author: Ganesh Nalawade (@ganeshrn)
short_description: Manage LLDP configuration on Cisco IOS network devices.
description:
- This module provides declarative management of LLDP service on Cisco IOS network
  devices.
version_added: 1.0.0
notes:
- Tested against IOS 15.2
options:
  state:
    description:
    - State of the LLDP configuration. If value is I(present) lldp will be enabled
      else if it is I(absent) it will be disabled.
    default: present
    choices:
    - present
    - absent
    - enabled
    - disabled
    type: str
extends_documentation_fragment:
- cisco.ios.ios
"""
EXAMPLES = """
- name: Enable LLDP service
  cisco.ios.ios_lldp:
    state: present

- name: Disable LLDP service
  cisco.ios.ios_lldp:
    state: absent
"""
RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always, except for the platforms that use Netconf transport to manage the device.
  type: list
  sample:
    - lldp run
"""
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    load_config,
    run_commands,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    ios_argument_spec,
)


def has_lldp(module):
    output = run_commands(module, ["show lldp"])
    is_lldp_enable = False
    if len(output) > 0 and "LLDP is not enabled" not in output[0]:
        is_lldp_enable = True
    return is_lldp_enable


def main():
    """ main entry point for module execution
    """
    argument_spec = dict(
        state=dict(
            default="present",
            choices=["present", "absent", "enabled", "disabled"],
        )
    )
    argument_spec.update(ios_argument_spec)
    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    warnings = list()
    result = {"changed": False}
    if warnings:
        result["warnings"] = warnings
    HAS_LLDP = has_lldp(module)
    commands = []
    if module.params["state"] == "absent" and HAS_LLDP:
        commands.append("no lldp run")
    elif module.params["state"] == "present" and not HAS_LLDP:
        commands.append("lldp run")
    result["commands"] = commands
    if commands:
        if not module.check_mode:
            load_config(module, commands)
        result["changed"] = True
    module.exit_json(**result)


if __name__ == "__main__":
    main()
